{**********************************************}
{                                              }
{              TNShellNotifyIcon               }
{                                              }
{        Copyright  2002 Tommy Nilsson        }
{  http://www.sharewareforce.com/tommynilsson  }
{                                              }
{**********************************************}

unit TNShellNotifyIcon;

interface

uses
  Classes, Controls, ExtCtrls, Forms, Graphics,
  Windows, Menus, Messages, ShellAPI;
  
type
  TShellMouseEvent = procedure(Sender: TObject; Button: TMouseButton) of object;

  TTNShellNotifyIcon = class(TComponent)
  private
    FActive, FAnimate: boolean;
    FAnimateInterval: integer;
    FAnimateImages: TImageList;
    FCurrentImage: word;
    FHint: string;
    FIcon, FOriginalIcon: TIcon;
    FIconData: TNotifyIconData;
    FIconList: TImageList;
    FTimer: TTimer;
    FVersion, FVersionDate: string;
    FOwnerHandle, FWindowHandle: HWND;
    FOnDblClick, FOnMouseDown, FOnMouseUp: TShellMouseEvent;
    FPopupMenu: TPopupMenu;
    procedure AddIcon;
    procedure AnimateList(value: boolean);
    procedure ChangeIcon(Sender: TObject);
    procedure DeleteIcon;
    procedure SetIcon(NewIcon:TIcon);
    procedure SetActive(const Value: boolean);
    procedure SetHint(const Value: string);
    procedure SetVersion(const Value: string);
    procedure SetVersionDate(const Value: string);
    procedure SetAnimate(const Value: boolean);
    procedure SetAnimateInterval(const Value: integer);
    procedure SetAnimateImages(const Value: TImageList);
    procedure WindowProc(var Message: TMessage);
    procedure SetPopupMenu(const Value: TPopupMenu);
    procedure ShowPopup;
  protected

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetIconFromIconList(index: integer);
  published
    property Active: boolean                read FActive           write SetActive;
    property Animate: boolean               read FAnimate          write SetAnimate;
    property AnimateInterval: integer       read FAnimateInterval  write SetAnimateInterval;
    property AnimateImages: TImageList      read FAnimateImages    write SetAnimateImages;
    property Hint: string                   read FHint             write SetHint;
    property IconList: TImageList           read FIconList         write FIconList;
    property PopupMenu: TPopupMenu          read FPopupMenu        write SetPopupMenu;
    property Version: string                read FVersion          write SetVersion;
    property VersionDate: string            read FVersionDate      write SetVersionDate;
    
    property OnDblClick: TShellMouseEvent   read FOnDblClick       write FOnDblClick;
    property OnMouseDown: TShellMouseEvent  read FOnMouseDown      write FOnMouseDown;
    property OnMouseUp: TShellMouseEvent    read FOnMouseUp        write FOnMouseUp;
  end;

var
  TNNotifyIcon: TTNShellNotifyIcon;
  
procedure Register;

implementation

const
  WM_ICONMESSAGE = WM_USER + 0;
  dateconst: string = 'October 15, 2002';
  versionconst: string = '1.0.1';
  
procedure Register;
begin
  RegisterComponents('Tommy', [TTNShellNotifyIcon]);
end;

{ TTNShellNotifyIcon }

constructor TTNShellNotifyIcon.Create(AOwner: TComponent);
begin
  inherited;
  FActive := false;
  FAnimate := false;
  FAnimateInterval := 100;
  FHint := '';
  FVersion := versionconst;
  FVersionDate := dateconst;
  FWindowHandle := AllocateHWnd(WindowProc);
  FIcon := TIcon.Create;
  FOriginalIcon := TIcon.Create;
  FTimer := TTimer.Create(Self);
  with FTimer do begin
    Enabled := false;
    OnTimer := ChangeIcon;
  end;
  FOwnerHandle := (Owner as TForm).Handle;
  if (csDesigning in ComponentState) then Exit;
    with FIconData do begin
      cbSize := SizeOf(TNotifyIconData);
      Wnd := FWindowHandle;
      uID := 0;
      uFlags := NIF_ICON or NIF_MESSAGE or NIF_TIP;
      uCallbackMessage := WM_ICONMESSAGE;
      hIcon := FIcon.Handle;
      CopyMemory(@FIconData.szTip, PChar(FHint), Length(FHint));
    end;
    if FActive then
      AddIcon;
end;

destructor TTNShellNotifyIcon.Destroy;
begin
  if FActive then
    DeleteIcon;
  FIcon.Free;
  FOriginalIcon.Free;
  FTimer.Free;
  if FWindowHandle<>0 then DeAllocateHWnd(FWindowHandle);
  inherited;
end;

procedure TTNShellNotifyIcon.AddIcon;
begin
  if FIcon.Empty then begin
    FIcon.Assign(Application.Icon);
    FIconData.hIcon:=FIcon.Handle;
  end;
  Shell_NotifyIcon(NIM_ADD, @FIconData);
end;

procedure TTNShellNotifyIcon.DeleteIcon;
begin
  if FAnimate then
    SetAnimate(false);
  Shell_NotifyIcon(NIM_DELETE, @FIconData);
end;

procedure TTNShellNotifyIcon.ChangeIcon(Sender: TObject);
var TempIcon: TIcon;
begin
  TempIcon:=TIcon.Create;
  try
    if FCurrentImage = Pred(FAnimateImages.Count) then
      FCurrentImage := 0
    else
      Inc(FCurrentImage);
    FAnimateImages.GetIcon(FCurrentImage, TempIcon);
    SetIcon(TempIcon);
  finally
    TempIcon.Free;
  end;
end;

procedure TTNShellNotifyIcon.AnimateList(value: boolean);
begin
  if ((Assigned(FAnimateImages)) and (FAnimateImages.Count>0)) then
    if value then begin
      FOriginalIcon.Assign(FIcon);
      FCurrentImage := 0;
      FTimer.Enabled := true;
    end else begin
      FTimer.Enabled := false;
      SetIcon(FOriginalIcon);
    end;
end;

procedure TTNShellNotifyIcon.SetIcon(NewIcon: TIcon);
begin
  FIcon.Assign(NewIcon);
  FIconData.hIcon:=FIcon.Handle;
  Shell_NotifyIcon(NIM_MODIFY, @FIconData);
end;

procedure TTNShellNotifyIcon.ShowPopup;                              
begin
  SetForegroundWindow(FOwnerHandle);
  FPopupMenu.Popup(Mouse.CursorPos.X,Mouse.CursorPos.Y);
  PostMessage(FOwnerHandle, WM_USER, 0, 0);
end;

procedure TTNShellNotifyIcon.SetIconFromIconList(index: integer);
var TempIcon: TIcon;
begin
  if index=(-1) then begin
    SetIcon(Application.Icon);
    Exit;
  end;

  if index<(-1) then
    Exit;
    
  TempIcon:=TIcon.Create;
  try
    if index<FIconList.Count then begin
      SetAnimate(false);
      FIconList.GetIcon(index, TempIcon);
      SetIcon(TempIcon);
    end;
  finally
    TempIcon.Free;
  end;
end;

procedure TTNShellNotifyIcon.WindowProc(var Message: TMessage);
begin
  case Message.Msg of
    WM_ICONMESSAGE: begin
      case Message.lParam of
        WM_LBUTTONDBLCLK: if Assigned(FOnDblClick)  then FOnDblClick(Self, mbLeft);
        WM_MBUTTONDBLCLK: if Assigned(FOnDblClick)  then FOnDblClick(Self, mbMiddle);
        WM_RBUTTONDBLCLK: if Assigned(FOnDblClick)  then FOnDblClick(Self, mbRight);
        WM_LBUTTONDOWN:   if Assigned(FOnMouseDown) then FOnMouseDown(Self, mbLeft);
        WM_MBUTTONDOWN:   if Assigned(FOnMouseDown) then FOnMouseDown(Self, mbMiddle);
        WM_RBUTTONDOWN:   if Assigned(FOnMouseDown) then FOnMouseDown(Self, mbRight);
        WM_LBUTTONUP:     if Assigned(FOnMouseUp)   then FOnMouseUp(Self, mbLeft);
        WM_MBUTTONUP:     if Assigned(FOnMouseUp)   then FOnMouseUp(Self, mbMiddle);
        WM_RBUTTONUP:     begin
                            if Assigned(FOnMouseUp) then FOnMouseUp(Self, mbRight);
                            if Assigned(FPopupMenu) then ShowPopup;
                          end;
      end;
    end;
  else Dispatch(Message);
  end;
end;

procedure TTNShellNotifyIcon.SetActive(const Value: boolean);
begin
  FActive := Value;
  if (csDesigning in ComponentState) then Exit;
  if FActive then
    AddIcon
  else
    DeleteIcon;
end;

procedure TTNShellNotifyIcon.SetHint(const Value: string);
begin
  if Length(Value)>64 then
    FHint := Copy(Value,1,64)
  else
    FHint := Value;
  if (csDesigning in ComponentState) then Exit;
  FHint := FHint+Chr(0);
  CopyMemory(@FIconData.szTip, PChar(FHint), Length(FHint));
  Shell_NotifyIcon(NIM_MODIFY, @FIconData);
end;

procedure TTNShellNotifyIcon.SetVersion(const Value: string);
begin
  FVersion := versionconst;
end;

procedure TTNShellNotifyIcon.SetVersionDate(const Value: string);
begin
  FVersionDate := dateconst;
end;

procedure TTNShellNotifyIcon.SetAnimate(const Value: boolean);
begin
  FAnimate := Value;
  AnimateList(FAnimate);
end;

procedure TTNShellNotifyIcon.SetAnimateInterval(const Value: integer);
begin
  FAnimateInterval := Value;
  FTimer.Interval := FAnimateInterval;
end;

procedure TTNShellNotifyIcon.SetAnimateImages(const Value: TImageList);
begin
  FAnimateImages := Value;
end;

procedure TTNShellNotifyIcon.SetPopupMenu(const Value: TPopupMenu);
begin
  FPopupMenu := Value;
end;

end.

